/*****************************************************************************

  Licensed to Accellera Systems Initiative Inc. (Accellera) under one or
  more contributor license agreements.  See the NOTICE file distributed
  with this work for additional information regarding copyright ownership.
  Accellera licenses this file to you under the Apache License, Version 2.0
  (the "License"); you may not use this file except in compliance with the
  License.  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
  implied.  See the License for the specific language governing
  permissions and limitations under the License.

 *****************************************************************************/

// sin_src from user's guide - extended with ac 

#include <systemc-ams>
#include "test_utilities.h"

SCA_TDF_MODULE(sin_src)
{
  sca_tdf::sca_out<double> out;
  
  sin_src( sc_core::sc_module_name nm, double offset_= 0.0, double ampl_= 1.0, 
           double noise_ampl_= 0.1, double freq_ = 1.0e3, 
           sca_core::sca_time Tm_ = sca_core::sca_time(0.125, sc_core::SC_MS) )
  : out("out"), offset(offset_), ampl(ampl_), noise_ampl(noise_ampl_), freq(freq_), Tm(Tm_)
  {}

  void set_attributes()
  {
    set_timestep(Tm);
  }

  void processing()
  {
    double t = get_time().to_seconds(); // actual time
    
    out.write( offset + ampl * std::sin( 2.0 * M_PI * freq * t ) );
  }
  
  void ac_processing()
  {
    if ( sca_ac_analysis::sca_ac_noise_is_running() )
      sca_ac_analysis::sca_ac_noise(out) = noise_ampl;
    else
      sca_ac_analysis::sca_ac(out) = ampl;
  }
 private:
  double offset, ampl, noise_ampl, freq; 
  sca_core::sca_time Tm;
};

int sc_main( int, char*[] )
{
  TEST_LABEL_START;

  sca_tdf::sca_signal<double> sig;
  
  sin_src src("src");

  src.out(sig);

  sca_util::sca_trace_file* tf = sca_util::sca_create_tabular_trace_file("sin_src");

  sca_util::sca_trace(tf, sig, "sig");

  sc_core::sc_start(3, sc_core::SC_MS);

  sca_util::sca_close_tabular_trace_file(tf);

  test_util::check_results("sin_src", 1, 1e-3);

  TEST_LABEL_END;
    
  return 0;
}
  
